"ams.rpm.register.lcms" <-
function(
	x, 
	y, 
	da.temp=1, 
	da.temp.min=0.002, 
	da.rate=0.4, 
	W=diag(c(1e2, 1e5, 1)), 
	out=0.05, 
	lambda.init, 
	max.points=min(nrow(x), nrow(y)), 
	sm.iterlim=5, 
	tie.method=c("none", "median", "noise"), 
	tie.sd=mean(diff(sort(unique(x[,1]))))/10,
	verbose=F
)
{
	stopifnot((ncol(x)==3) && (ncol(y)==3));
	# intensity normalization: minmax
	x.int.min <- min(x[,3]);
	x.int.max <- max(x[,3]);
	x[,3] <- (x[,3]-x.int.min) / (x.int.max-x.int.min);
	y.int.min <- min(y[,3]);
	y.int.max <- max(y[,3]);
	y[,3] <- (y[,3]-y.int.min) / (y.int.max-y.int.min);
	# sort by rt
	x <- x[sort(x[,1], index.return=T)$ix,];
	y <- y[sort(y[,1], index.return=T)$ix,];
	# reserve space for the correspondence matrix
	x.N <- nrow(x);
	y.N <- nrow(y);
	cm <- matrix(0, x.N+1, y.N+1);
	# our estimate of y (in dimensions of x)
	yhat <- x;
	# smoothness / df control
	lambda <- lambda.init;

	while(da.temp > da.temp.min) {
		if (verbose) print(paste("[", da.temp, "/", da.temp.min, "]: lambda=", lambda));
		
	     ###### correspondence estimation 
		cm[1:x.N, 1:y.N] <- exp(xdist(yhat, y, w=W)^2 / (-2*da.temp))/da.temp;		
		cm[1:x.N, y.N+1] <- out;
		cm[x.N+1, 1:y.N] <- out;
		# normalize correspondence matrix
		cm <- sinkhorn(cm, tol=1e-20, max.it=30);

	     ###### smooth monotone regression ###
		# we ignore intensity information here, so drop it
		yhat.t.rt <- cm[1:x.N, 1:y.N] %*% y[,1]; 

		# get rid of all points which correspond to outliers,
		# i.e. valid rows do not have their maximum in the outlier column
		yhat.valid.rows <- which(
			apply(cm[1:x.N,], 1, which.max)!=y.N+1
		);
		
		if (tie.method %in% c("none", "noise")) {
			if (length(yhat.valid.rows) > max.points) {
				# for subselections, just take the N points with the best correspondences
				best.idx <- sort(
					apply(cm[1:x.N, 1:y.N], 1, max),
					decreasing=T,
					index.return=T
				)$ix;
				best.idx.valid <- best.idx[best.idx %in% yhat.valid.rows];
				yhat.valid.rows <- best.idx.valid[1:min(max.points,length(best.idx.valid))];
			}
		}		
		if (verbose) { print(paste("#valid points:", length(yhat.valid.rows))); }
		x.rt.valid <- x[yhat.valid.rows,1]
		wgt.valid <- apply(cm[1:x.N, 1:y.N], 1, max)[yhat.valid.rows];

		# problem: sinkhorn() may not converge to 1, hence
		# divide each row by row sum of CM
		yhat.rowsums <- apply(cm[1:x.N, 1:y.N], 1, sum)[yhat.valid.rows];
		yhat.t.rt.valid <- yhat.t.rt[yhat.valid.rows]/yhat.rowsums;
		
		# spline interpolation problem: we must not throw away the first or last point.
		# if this is an outlier, re-add it, and modify yhat.t.rt.valid to have the
		# point on the diagonal *with a very small weight*
		if (!(which.min(x[,1]) %in% yhat.valid.rows)) {
			yhat.valid.rows <- c(which.min(x[,1]), yhat.valid.rows);
			yhat.t.rt.valid <- c(min(x[,1]), yhat.t.rt.valid);
			x.rt.valid <- c(min(x[,1]), x.rt.valid);
			wgt.valid <- c(1e-8, wgt.valid);
		}
		if (!(which.max(x[,1]) %in% yhat.valid.rows)) {
			yhat.valid.rows <- c(yhat.valid.rows, which.max(x[,1]));
			yhat.t.rt.valid <- c(yhat.t.rt.valid, max(x[,1]));
			x.rt.valid <- c(x.rt.valid, max(x[,1]));
			wgt.valid <- c(wgt.valid, 1e-8);
		}
		if (tie.method == "median") {
			# method 1: replace by median
			# smooth.monotone gets *really* slow with multiple observations at coinciding
			# values of the explanatory variable, so we use rt times as factors to calculate
			# an rt-wise median, along with the corresponding weight (as max weight 
			# at a specific rt)
			yhat.t.rt.valid <- as.numeric(tapply(yhat.t.rt.valid, as.factor(x.rt.valid), median))
			x.rt.valid <- unique(x.rt.valid); 
			wgt.valid <- as.numeric(tapply(yhat.t.rt.valid, as.factor(x.rt.valid), max))
			if (verbose) { print(paste("# unique x.rt points:", length(x.rt.valid))); }
		}

		if (tie.method == "noise") {
			# method 2: add a tiny bit of noise
			x.rt.valid <- c(x.rt.valid[1],
				x.rt.valid[2:(length(x.rt.valid)-1)]+
					rnorm(length(x.rt.valid)-2, mean=0, sd=tie.sd),
				x.rt.valid[length(x.rt.valid)]
				);
		}
		norder <- 6;
        	range <- range(x.rt.valid);
        	nbasis <- length(x.rt.valid)+norder-2; # spline knot at each observation
        	wbasis <- create.bspline.basis(range, nbasis, norder, x.rt.valid);
        	cvec0 <- matrix(0, nbasis, 1);
        	Wfd0 <- fd(cvec0, wbasis);
        	Lfdobj <- 3;
        	dat2fdPar <- fdPar(Wfd0, Lfdobj, lambda);
		result <- smooth.monotone(
			x.rt.valid,
			yhat.t.rt.valid, 
			wgt.valid, 
			dat2fdPar, 
			iterlim=sm.iterlim, 
			dbglev=1
		);
        	yhat[,1] <- result$beta[1] + result$beta[2]*eval.monfd(x[,1], result$Wfd);
		if (verbose) {
			plot(x[,1], yhat[,1], col="red", type="l");
			points(x.rt.valid, yhat.t.rt.valid);
		}
	     ###### update annealing parameters		
		da.temp <- da.temp*da.rate;
		lambda <- lambda.init * da.temp;
	}
	# rescale intensity
	yhat[,3] <- yhat[,3]*(x.int.max-x.int.min) + x.int.min;
	ret <- list(
			yhat = yhat,
			warping = result
	);
	class(ret) <- "ams.rpm.2d";
	return(ret);
}

