"ams.rpm.register.tic" <-
function(
	x, 
	y, 
	da.temp=1, 
	da.temp.min=0.002, 
	da.rate=0.4, 
	W=diag(c(10,100)), 
	kappa=c(0.1, 0.01), 
	out=0.05, 
	lambda.init=1e-10, 
	max.points=nrow(x), 
	subsample.method=c("equidistant", "derivative"),
	sm.iterlim=5,
	verbose=F
) 
{
	stopifnot((ncol(x)==2) && (ncol(y)==2));
	# intensity normalization: minmax
	x.int.min <- min(x[,2]);
	x.int.max <- max(x[,2]);
	x[,2] <- (x[,2]-x.int.min) / (x.int.max-x.int.min);
	y.int.min <- min(y[,2]);
	y.int.max <- max(y[,2]);
	y[,2] <- (y[,2]-y.int.min) / (y.int.max-y.int.min);
	# we register x onto y. Hence, y's derivative will never change
	y.D <- c(0, diff(y[,2]) / diff(y[,1]));
	# reserve space for the correspondence matrix
	x.N <- nrow(x);
	y.N <- nrow(y);
	cm <- matrix(0, x.N+1, y.N+1);
	# our estimate of y (in dimensions of x)
	yhat <- x;
	# smoothness / df control
	lambda <- lambda.init;
	# warping result (declared here, so we can return it)
	result <- NULL;

	while(da.temp > da.temp.min) {
		if (verbose) print(paste("[", da.temp, "/", da.temp.min, "]: lambda=", lambda));
		
		### correspondence estimation 
		yhat.D <- c(0, diff(yhat[,2]) / diff(yhat[,1])); # speedup possible here
		cm[1:x.N, 1:y.N] <- exp(
				(xdist(yhat, y, w=W) + 
				kappa[1] * abs(xdist(yhat.D, y.D)) +
				kappa[2] * as.numeric(xdist(sign(yhat.D),sign(y.D))>0))^2 / (-2*da.temp)
			)/da.temp;		
		cm[1:x.N, y.N+1] <- out;
		cm[x.N+1, 1:y.N] <- out;
		# normalize correspondence matrix
		cm <- sinkhorn(cm, tol=1e-20, max.it=30);

		### smooth monotone regression ###
		# we ignore intensity information here, so drop it
		yhat.t.rt <- cm[1:x.N, 1:y.N] %*% y[,1]; 

		# get rid of all points which correspond to outliers,
		# i.e. valid rows do not have their maximum in the outlier column
		yhat.valid.rows <- which(
			apply(cm[1:x.N,], 1, which.max)!=y.N+1
		);
		
		if (subsample.method=="derivative") {
			yhat.smoothed <- smooth.spline(yhat[,1], yhat[,2]);
			yhat.smoothed.D <- c(diff(yhat.smoothed$y)/diff(yhat.smoothed$x));
			yhat.smoothed.DD <- c(yhat.smoothed.D/diff(yhat.smoothed$x));
			yhat.subselections <- unique(c(
				as.numeric(which(diff(sign(yhat.smoothed.D))!=0)),
				as.numeric(which(diff(sign(yhat.smoothed.DD))!=0))
			));
			yhat.valid.rows <- intersect(yhat.valid.rows, yhat.subselections);
		}
		if (subsample.method=="equidistant") {
			yhat.subselections <- floor(
				seq(1,length(yhat.t.rt),length.out=min(max.points, length(yhat.t.rt)))
			);
			yhat.valid.rows <- intersect(yhat.valid.rows, yhat.subselections);
		}
		if (verbose) { print(paste("#valid points:", length(yhat.valid.rows))); }
		x.rt.valid <- x[yhat.valid.rows,1]
		wgt.valid <- apply(cm[1:x.N, 1:y.N], 1, max)[yhat.valid.rows];

		# problem: sinkhorn() may not converge to 1, hence
		# divide each row by row sum of CM
		yhat.rowsums <- apply(cm[1:x.N, 1:y.N], 1, sum)[yhat.valid.rows];
		yhat.t.rt.valid <- yhat.t.rt[yhat.valid.rows]/yhat.rowsums;
		
		# spline interpolation problem: we must not throw away the first or last point.
		# if this is an outlier, re-add it, and modify yhat.t.rt.valid to have the
		# point on the diagonal *with a very small weight*
		if (!(1 %in% yhat.valid.rows)) {
			yhat.valid.rows <- c(1, yhat.valid.rows);
			yhat.t.rt.valid <- c(x[1,1], yhat.t.rt.valid);
			x.rt.valid <- c(x[1,1], x.rt.valid);
			wgt.valid <- c(1e-8, wgt.valid);
		}
		if (!(length(yhat.t.rt) %in% yhat.valid.rows)) {
			yhat.valid.rows <- c(yhat.valid.rows, length(yhat.t.rt));
			yhat.t.rt.valid <- c(yhat.t.rt.valid, x[length(yhat.t.rt),1]);
			x.rt.valid <- c(x.rt.valid, x[length(yhat.t.rt),1]);
			wgt.valid <- c(wgt.valid, 1e-8);
		}

		norder <- 6;
        	range <- c(x.rt.valid[1], x.rt.valid[length(x.rt.valid)]);
        	nbasis <- length(x.rt.valid)+norder-2; # spline knot at each observation
        	wbasis <- create.bspline.basis(range, nbasis, norder, x.rt.valid);
        	cvec0 <- matrix(0, nbasis, 1);
        	Wfd0 <- fd(cvec0, wbasis);
        	Lfdobj <- 3;
        	dat2fdPar <- fdPar(Wfd0, Lfdobj, lambda);
		result <- smooth.monotone(
			x.rt.valid,
			yhat.t.rt.valid, 
			wgt.valid, 
			dat2fdPar, 
			iterlim=sm.iterlim, 
			dbglev=1
		);
        	yhat[,1] <- result$beta[1] + result$beta[2]*eval.monfd(x[,1], result$Wfd);

		### update annealing parameters		
		da.temp <- da.temp*da.rate;
		lambda <- lambda.init * da.temp;
	}
	# rescale intensity
	yhat[,2] <- yhat[,2]*(x.int.max-x.int.min) + x.int.min;
	ret <- list(
		yhat = yhat,
		warping = result
	);
	class(ret) <- "ams.rpm.1d";
	return(ret);
}

