\name{ams.rpm.register}
\alias{ams.rpm.register.tic}
\alias{ams.rpm.register.lcms}
\title{ Retention Time Alignment of LC/MS data }
\description{
	These are the retention time alignment workhorse
	functions. \code{ams.rpm.register.tic} is used to
	align TIC chromatograms, \code{ams.rpm.register.lcms}
	will align data sets where mass/charge information is
	available.
}
\usage{
ams.rpm.register.tic(x, y, da.temp = 1, da.temp.min = 0.002, 
	da.rate = 0.4, W = diag(c(10, 100)), kappa = c(0.1, 0.01),
	out = 0.05, lambda.init = 1e-10, max.points = nrow(x), 
	subsample.method = c("equidistant", "derivative"), 
	sm.iterlim = 5, verbose = F)

ams.rpm.register.lcms(x, y, da.temp = 1, da.temp.min = 0.002, 
	da.rate = 0.4, W = diag(c(100, 1e+05, 1)), out = 0.05, 
	lambda.init, max.points = min(nrow(x), nrow(y)), sm.iterlim = 5, 
	tie.method = c("none", "median", "noise"), 
	tie.sd = mean(diff(sort(unique(x[, 1]))))/10, verbose = F)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
  	Point set in matrix format, one point per row.
	The number of columns should be 2 (rt, intensity) for 
	\code{ams.rpm.register.tic} and 3 (rt, m/z, intensity) for
	\code{ams.rpm.register.tic}. The point set \code{x} is
	aligned to \code{y}.
  }
  \item{y}{
  	Point set \code{x} is aligned to. The format equals
	the one described for \code{x}.
  }
  \item{da.temp}{
  	Both registration procedures use a deterministic
	annealing scheme, gradually allowing more flexible
	registrations. This parameter controls the initial
	annealing temperature.  	
  }
  \item{da.temp.min}{
  	The annealing temperature threshold.   	
  }
  \item{da.rate}{
  	The factor by which the annealing rate is decreased
	in each cycle. 
  }
  \item{W}{
  	Weight matrix controlling the contributions
	of the point set dimensions (rt, m/z, intensity)
	to the dissimilarity measure.
  }
  \item{kappa}{
  	For \code{ams.register.tic} only, this parameter
	controls the contribution of the derivative slope 
	and sign information.
  }
  \item{out}{
  	Prior probability for a point being an outlier.
	If this becomes a points larges correspondence
	matrix entry, it is excluded from the transformation
	estimation.
  }
  \item{lambda.init}{
  	The initial value for smoothness control. Lambda is
	decreased alogn with the annealing temperature.
  }
  \item{max.points}{
  	The maximum number of points to be used in the call
	to \code{smooth.monotone}. This provides a way to 
	limit the runtime of large calculations (at the price
	of accuracy, of course).
  }
  \item{subsample.method}{
  	For \code{ams.register.tic} only. As TICs tend to be
	continious point sets, it is not necessary to put
	spline knots on every point. Selecting 
	\code{equidistant} will subsample the signal 
	equidistantly, yielding \code{max.points} points
	as spline knots. Selecting \code{derivative}, will 
	try to select points with first and second derivatives
	of a smoothed version of \code{x} equal to zero.
  }
  \item{sm.iterlim}{
  	This parameter provides a way to limit the number of
	Newton-Raphson descent steps \code{smooth.monotone}
	will attempt to carry out. Small values will increase
	speed at the expense og accuracy.
  }
  \item{tie.method}{
  	For \code{ams.register.lcms} only.  In 
	the full LC/MS case, we are likely to observe multiple 
	points eluting at the same retention time.
	This slows down the calculation considerably.
	For the transformation estimation with \code{smooth.monotone}
	the \code{median} method replaces all points with 
	equal retention times in \code{x} by single points
	(the estimate medians). The \code{noise} method will
	add a tiny amount of noise (controlled by \code{tie.sd}
	on the retention times.
  }
  \item{tie.sd}{
  	The standard deviation of the zero-mean normal noise
	added on \code{x} retention times for transformation
	estimation. This parameter only effects the calculation
	if \code{tie.method=="noise"}.
  }
  \item{verbose}{
  	Verbosity switch. \code{TRUE} will cause diagnostic messages
	to be printed.
  }
}
\details{
	The mathematical details are described in the references.
}
\value{
  An \emph{ams.rpm.1d} object for TIC chromatogram alignment, or
  an \emph{ams.rpm.2d} object in the full LC/MS alignment case.
  Slots are
  \item{yhat}{
  	The result of warping point set \code{x}.
  }
  \item{warping}{
  	The \code{smooth.monotone} result object}
  }
}
\references{
	Kirchner M, Saussen B, Steen H, Steen JAJ, Hamprecht FA, "amsrpm: Robust Point Matching for Retention Time Alignment of LC/MS Data in R", Journal of Statistical Software, submitted (2006).
}
\author{
	Marc Kirchner \email{marc.kirchner@iwr.uni-heidelberg.de}
	Benjamin Saussen \email{benajmin.saussen@iwr.uni-heidelberg.de}
}
%\note{ ~~further notes~~ }
\examples{
\dontrun{
# TIC data
data(pho4_tic)
registration <- ams.rpm.register.tic(
		pho4_tic[[1]],
		pho4_tic[[2]], 
		kappa=c(0.1, 0.01),
		max.points=150,
		subsample.method="equidistant")
ams.plot(res, pho4_tic[[1]], pho4_tic[[2]])
}
\dontrun{
# LC/MS data
res <- ams.rpm.register.lcms(
            x=lcms.1,
            y=lcms.2,
            max.points=200,
            tie.method="noise")
ams.plot(res, lcms.1, lcms.2)
}
}
\seealso{ \code{\link[fda]{smooth.monotone}} }
\keyword{ multivariate }
